@@ -88,6 +88,7 @@ brw_emit_sampler_state(struct brw_context *brw,
                        unsigned wrap_s,
                        unsigned wrap_t,
                        unsigned wrap_r,
+                       unsigned base_level,
                        unsigned min_lod,
                        unsigned max_lod,
                        int lod_bias,
@@ -132,6 +133,21 @@ brw_emit_sampler_state(struct brw_context *brw,
       ss[0] |= SET_FIELD(lod_bias & 0x7ff, GEN4_SAMPLER_LOD_BIAS) |
                SET_FIELD(shadow_function, GEN4_SAMPLER_SHADOW_FUNCTION);
 
+      /* This field has existed since the original i965, but is declared MBZ
+       * until Sandy Bridge.  According to the PRM:
+       *
+       *    "This was added to match OpenGL semantics"
+       *
+       * In particular, OpenGL allowed you to offset by 0.5 in certain cases
+       * to get slightly better filtering.  On Ivy Bridge and above, it
+       * appears that this is added to RENDER_SURFACE_STATE::SurfaceMinLOD so
+       * the right value is 0.0 or 0.5 (if you want the wacky behavior).  On
+       * Sandy Bridge, however, this sum does not seem to occur and you have
+       * to set it to the actual base level of the texture.
+       */
+      if (brw->gen == 6)
+         ss[0] |= SET_FIELD(base_level, BRW_SAMPLER_BASE_MIPLEVEL);
+
       if (brw->gen == 6 && min_filter != mag_filter)
          ss[0] |= GEN6_SAMPLER_MIN_MAG_NOT_EQUAL;
 
@@ -149,7 +165,7 @@ brw_emit_sampler_state(struct brw_context *brw,
 static uint32_t
 translate_wrap_mode(struct brw_context *brw, GLenum wrap, bool using_nearest)
 {
-   switch( wrap ) {
+   switch (wrap) {
    case GL_REPEAT:
       return BRW_TEXCOORDMODE_WRAP;
    case GL_CLAMP:
@@ -393,8 +409,7 @@ brw_update_sampler_state(struct brw_context *brw,
                          GLenum target, bool tex_cube_map_seamless,
                          GLfloat tex_unit_lod_bias,
                          mesa_format format, GLenum base_format,
-                         bool is_integer_format,
-                         bool is_stencil_sampling,
+                         const struct gl_texture_object *texObj,
                          const struct gl_sampler_object *sampler,
                          uint32_t *sampler_state,
                          uint32_t batch_offset_for_sampler_state)
@@ -477,7 +492,7 @@ brw_update_sampler_state(struct brw_context *brw,
        * integer formats.  Fall back to CLAMP for now.
        */
       if ((tex_cube_map_seamless || sampler->CubeMapSeamless) &&
-          !(brw->gen == 7 && !brw->is_haswell && is_integer_format)) {
+          !(brw->gen == 7 && !brw->is_haswell && texObj->_IsIntegerFormat)) {
 	 wrap_s = BRW_TEXCOORDMODE_CUBE;
 	 wrap_t = BRW_TEXCOORDMODE_CUBE;
 	 wrap_r = BRW_TEXCOORDMODE_CUBE;
@@ -503,8 +518,13 @@ brw_update_sampler_state(struct brw_context *brw,
    }
 
    const int lod_bits = brw->gen >= 7 ? 8 : 6;
-   const unsigned min_lod = U_FIXED(CLAMP(sampler->MinLod, 0, 13), lod_bits);
-   const unsigned max_lod = U_FIXED(CLAMP(sampler->MaxLod, 0, 13), lod_bits);
+   const float hw_max_lod = brw->gen >= 7 ? 14 : 13;
+   const unsigned base_level =
+      U_FIXED(CLAMP(texObj->MinLevel + texObj->BaseLevel, 0, hw_max_lod), 1);
+   const unsigned min_lod =
+      U_FIXED(CLAMP(sampler->MinLod, 0, hw_max_lod), lod_bits);
+   const unsigned max_lod =
+      U_FIXED(CLAMP(sampler->MaxLod, 0, hw_max_lod), lod_bits);
    const int lod_bias =
       S_FIXED(CLAMP(tex_unit_lod_bias + sampler->LodBias, -16, 15), lod_bits);
 
@@ -517,7 +537,7 @@ brw_update_sampler_state(struct brw_context *brw,
        wrap_mode_needs_border_color(wrap_t) ||
        wrap_mode_needs_border_color(wrap_r)) {
       upload_default_color(brw, sampler, format, base_format,
-                           is_integer_format, is_stencil_sampling,
+                           texObj->_IsIntegerFormat, texObj->StencilSampling,
                            &border_color_offset);
    }
 
@@ -530,7 +550,7 @@ brw_update_sampler_state(struct brw_context *brw,
                           max_anisotropy,
                           address_rounding,
                           wrap_s, wrap_t, wrap_r,
-                          min_lod, max_lod, lod_bias,
+                          base_level, min_lod, max_lod, lod_bias,
                           shadow_function,
                           non_normalized_coords,
                           border_color_offset);
@@ -555,8 +575,7 @@ update_sampler_state(struct brw_context *brw,
    brw_update_sampler_state(brw, texObj->Target, ctx->Texture.CubeMapSeamless,
                             texUnit->LodBias,
                             firstImage->TexFormat, firstImage->_BaseFormat,
-                            texObj->_IsIntegerFormat, texObj->StencilSampling,
-                            sampler,
+                            texObj, sampler,
                             sampler_state, batch_offset_for_sampler_state);
 }
 
